/**
 * @file dmxplayer.h
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * To play and record dmx programs
**/
#ifndef DMXPLAYER_H
#define DMXPLAYER_H
#include "dmxprogram.h"
#include <stdio.h>

/* number of programs available */
#define DMXPLAYER_PROGRAMS		99
/* frames per second */
#define DMXPLAYER_FPS			30
/* Timeout for tapping in seconds */
#define DMXPLAYER_TAPTIMEOUT	3


/* initiate */
int dmxplayer_init();
/* clean-up and destruction */
void dmxplayer_destroy();
/* render a scene - called by timer started by init, dont use */
void dmxplayer_render(long);
/* set a programmer channel */
int dmxplayer_set_prog(int, int);
/* set a programmer channel additive */
int dmxplayer_set_prog_add(int, int);
/* unset a programmer channel */
int dmxplayer_unset_prog(int);
/* clear the whole programmer */
int dmxplayer_clear_prog();
/* get a channel from programmer if existing */
int dmxplayer_get_prog_value(int);
/* true if some programmer values are set */
int dmxplayer_is_set_prog();
/* store a program */
int dmxplayer_store_program(int);
/* toogle programs on and off */
int dmxplayer_toogle_program(int);
/* start a program */
int dmxplayer_start_program(int);
/* stop a program */
int dmxplayer_stop_program(int);
/* sync */
void dmxplayer_timesync();
/* sync and time meter */
int dmxplayer_timetap();
/* Set beats per minute */
void dmxplayer_setbpm(float);
/* get current beats per minute */
float dmxplayer_getbpm();
/* get current speed as seconds - JUST USE FOR GUI OUTPUT */
float dmxplayer_get_speed_sec();
/* get a program by index */
dmxprogram_t *dmxplayer_getprogram(int);
/* saves into a file */
long dmxplayer_save(FILE*);
/* loads from a file */
long dmxplayer_load(FILE*);
/* true, if tap is timed out */
int dmxplayer_istaptimeout();
/* returns quarter counter */
long dmxplayer_getQuarterCnt();


#endif /* DMXPLAYER_H */
