/**
 * @file dmxgui_tools.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Jan-2013 / All rights reserved. 
 * ---------------------------------------------------------
 * Provides gui tools
**/
#include "dmxgui_tools.h"
#include <stdlib.h>


/**
 * dmxgui_ncurses_settings()
 * Set global ncurses settings
**/
void dmxgui_ncurses_settings()
{
	initscr();
	cbreak();
	noecho();
	raw();
	halfdelay(1);
	keypad(stdscr,TRUE);
}


/**
 * dmxgui_color_init()
 * Initiate and set colors
**/
void dmxgui_color_init()
{
	/* ncurses color init */
	start_color();
	/* border and information */
	init_pair(1, COLOR_YELLOW, COLOR_BLACK);
	/* channel text */
	init_pair(2, COLOR_GREEN,   COLOR_BLACK);
	/* error text */
	init_pair(3, COLOR_RED,    COLOR_BLACK);
	/* channel user input */
	init_pair(4, COLOR_GREEN,  COLOR_YELLOW);
	/* global stuff */
	init_pair(5, COLOR_MAGENTA,  COLOR_BLACK);
}


/**
 * dmxgui_print_key_bindings()
 * Prints key bindings into window
**/
void dmxgui_print_key_bindings(WINDOW *wnd)
{
	/* show information */
	wattron(wnd, COLOR_PAIR(1));
	mvwprintw(wnd, LINES-4, 2, "[2*ESC] Quit  |  [F8] Sync  |  [SPACE] Tap Sync");
	mvwprintw(wnd, LINES-3, 2, "[SHIFT+U] Unset Channel  |  [SHIFT+C] Clear Programmer  |  [SHIFT+S] Store");
	mvwprintw(wnd, LINES-2, 2, "[PGDWN] -10  |  [PGUP] +10  |  [-] -1  |  [+] +1  |  [TAB] View  |  [,] Prev  |  [.] Next");
	wattroff(wnd, COLOR_PAIR(1));
}



/**
 * dmxgui_get_step_sign()
 * Gets a sign for 4 steps
 * step is taken mod 4
**/
char dmxgui_get_step_sign(int step)
{
	switch(step%4)
	{
		case 0:	
			return '|';
		case 1:
			return '/';
		case 2:
			return '-';
		case 3:
			return '\\';
		default:
			break;
	}
	return '~';
}



/**
 * dmxgui_get_step_sign()
 * Prints active tab title line
 * TODO: BETTER IMPLEMENTATION
**/
void dmxgui_print_tab_titles(WINDOW* wnd, int active)
{
	mvwprintw(wnd, 0, 2,  "DMXPROG");
	switch(active)
	{
		case 0:
			mvwprintw(wnd, 0, 11, "[ >Programmer< |  Chasers  |  Detail  |  Edit  |  Global  ]");
			break;
		case 1:
			mvwprintw(wnd, 0, 11, "[  Programmer  | >Chasers< |  Detail  |  Edit  |  Global  ]");
			break;
		case 2:
			mvwprintw(wnd, 0, 11, "[  Programmer  |  Chasers  | >Detail< |  Edit  |  Global  ]");
			break;
		case 3:
			mvwprintw(wnd, 0, 11, "[  Programmer  |  Chasers  |  Detail  | >Edit< |  Global  ]");
			break;
		case 4:
			mvwprintw(wnd, 0, 11, "[  Programmer  |  Chasers  |  Detail  |  Edit  | >Global< ]");
			break;
		default:
			mvwprintw(wnd, 0, 11, "[  Programmer  |  Chasers  |  Detail  |  Edit  |  Global  ]");
			break;
	}
}




/**
 * dmxgui_clear_line
 * Clears a line
**/
void dmxgui_clear_line(WINDOW* wnd, int line)
{
	mvwhline(wnd, line, 1, ' ', COLS-2);
}
