/**
 * @file miditable.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Creates the midi table
**/
#include "miditable.h"
#include "../dmxstr.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/**
 * Initiate a midi table
 * @return Pointer to a new table
**/
midi_table_t *miditable_init()
{
	int n;
	midi_table_t *table = (midi_table_t*) malloc( sizeof(midi_table_t) );
	/* create table */
	for(n=0;n<255;n++)
	{
		table->data[n] = 0;
	}
	return table;
}



/**
 * Add something to a midi table
**/
void miditable_add(midi_table_t *table, midib byte1, midib byte2, const char *cmd)
{
	int n;
	/* check if row exists */
	if( table->data[byte1] == 0 ) 
	{
		table->data[byte1] = (char**) malloc( sizeof(char*) * 255 );
		for(n=0;n<255;n++)
		{
			table->data[byte1][n] = 0;
		}
	}
	/* check if already assigned */
	if( table->data[byte1][byte2] != 0 )
	{
		free( table->data[byte1][byte2] );
	}
	/* remove leading whitespace */
	cmd = dmxstr_remove_whitespace(cmd);
	/* allocate and set command */
	table->data[byte1][byte2] = (char*) malloc(  sizeof(char) * (strlen(cmd)+1) );
	strcpy(table->data[byte1][byte2], cmd);
}



/**
 * Initiate a midi table
 * @param table  The table to destroy
**/
const char *miditable_get_cmd(midi_table_t *table, midib byte1, midib byte2)
{
	/* check table */
	if(table == 0) return 0;
	/* check if row exists */
	if( table->data[byte1] == 0 ) 
		return 0;
	/* check if already assigned */
	if( table->data[byte1][byte2] == 0 )
	{
		return 0;
	}
	/* return cmd string */
	return table->data[byte1][byte2];
}



/**
 * Initiate a midi table
 * @param table  The table to destroy
**/
void miditable_destroy(midi_table_t *table)
{
	int n, i;
	/* iterate rows */
	for(n=0;n<255;n++)
	{
		if(table->data[n] == 0) continue;
		/* iterate entries */
		for(i=0;i<255;i++)
		{
			if(table->data[n][i] == 0) continue;
			free( table->data[n][i] );
		}
		free( table->data[n] );
	}
	/* free table itself */
	free(table);
}


