/**
 * @file lindxmd_tcp.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / 2012-2014 / All rights reserved. 
 * ---------------------------------------------------------
 * Here the tcp service is done.
**/
#include "cmdexec.h"
#include "lindmxd.h"
#include "lindmxd_tcp.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netinet/in.h>


/* global variables */
int stopsrv      = 0;
int connections  = 0;
int srv_sock     = 0;

/**
 * lindmxd_thread_client()
 * The client thread
 * @param arg Pointer on the client socket
**/
void *lindmxd_thread_client(void *arg)
{
	int cli_sock = *(int *)arg;
	char buf[LINDMXD_BUFLEN];
	connections++;
	/* Main loop until break or until error while reading/writing */
	int n = 1;
	int ret;
	while(n > 0)
	{
		/* Receive message from client */
		n = read(cli_sock, buf, LINDMXD_BUFLEN);
		if(n<=0) break;
		/* Server was already down */
		if( stopsrv ) 
		{
			n = write(cli_sock, "out of service\r\n", 16);
			break;
		}
		/* transform \r\n ending to \0 */
		kill_lineend(buf, LINDMXD_BUFLEN);

		ret = cmdexec_do(buf, buf, LINDMXD_BUFLEN);
		n = write(cli_sock, buf, strlen(buf));
		if(ret == CMDEXEC_CLIQUIT) break;
	}

	/* decrease connection counter */
	connections--;
	/* Close client socket */
	close(cli_sock);
	syslog(LOG_DEBUG, "tcp client quit (%d)", cli_sock);

	return NULL;
} 



/**
 * lindmxd_run()
 * This will start and run the LINDMX-Daemon
 * @param port        Port to run the server on
*/
void lindmxd_run_tcp(int port)
{
	/* Server socket */
	int cli_sock;
	struct sockaddr_in srv_addr, cli_addr;

	/* Create server socket */
	syslog(LOG_DEBUG, "creating a new server socket");
	srv_sock = socket(AF_INET, SOCK_STREAM, 0);
	if(srv_sock < 0) 
	{
     	syslog(LOG_ERR, "[ERROR] creating the servers socket!");
		exit(EXIT_FAILURE);
	}

	/* Initialize server socket address */
	bzero((char *) &srv_addr, sizeof(srv_addr));
	srv_addr.sin_family       = AF_INET;
	srv_addr.sin_addr.s_addr  = INADDR_ANY;
	srv_addr.sin_port         = htons(port);

	/* Bind address to server socket */
	syslog(LOG_DEBUG, "binding tcp socket on port %d", port);
	if( bind(srv_sock, (struct sockaddr *) &srv_addr, sizeof(srv_addr)) < 0 )
	{
		close(srv_sock);
		syslog(LOG_ERR, "[ERROR] not able to bind address to the server socket!");
		exit(EXIT_FAILURE);
	}
	syslog(LOG_NOTICE, "TCP instance listening on port %d", port);

	/* main loop */
	while( 1 )
	{
		char client_ip[50];
		/* thread for the client connection */
		pthread_t child_thread;
		/* begin listening */
		listen(srv_sock, 5);

		/* get the new client */
		socklen_t clilen = sizeof(cli_addr);
		cli_sock = accept(srv_sock, (struct sockaddr *) &cli_addr, &clilen);
		if(cli_sock < 0)
		{
		 	syslog(LOG_ERR, "[ERROR] accepting a client");
			continue;
		}
		inet_ntop(AF_INET, &cli_addr, client_ip, 50);
		syslog(LOG_DEBUG, "accepted tcp:%d client (%d)", port, cli_sock);

		/* create child thread */
		syslog(LOG_DEBUG, "creating client thread (cli: %d)", cli_sock);
		pthread_create(&child_thread, NULL, lindmxd_thread_client, (void*) &cli_sock);
		/* kill resources when quit / return value not needed */
		pthread_detach(child_thread);
	}

	abort();
}
